"""
Created on Oct 30, 2013

@author: Nicklas Boerjesson
"""
from tkinter import Frame, SUNKEN, StringVar, IntVar, ttk, YView, XView, Scrollbar, Canvas, \
    Variable, Text, Button
from tkinter.constants import E, W, LEFT, RIGHT, CENTER, TOP, BOTTOM, X, VERTICAL, FALSE, Y, BOTH, TRUE, NW, END


def make_entry(_master, _caption, _row, _split_sticky=None, _button_caption= None):
    """
    Creates a label, a property variable and text edit box(Entry) and sets stickiness.
    :param _master: The master widget
    :param _caption: The caption for the label
    :param _row: On what row it should be
    :param _split_sticky: Stick together or be apart.
    :return: Returns _variable (the property variable), _entry (the edit box) and _label (the label),
    """
    _label = ttk.Label(_master, text=_caption)
    _label.grid(column=0, row=_row, sticky=W)
    _variable = StringVar()
    _entry = ttk.Entry(_master, textvariable=_variable)
    if _split_sticky:
        _entry.grid(column=1, row=_row, sticky=E)
    else:
        _entry.grid(column=1, row=_row, sticky=W)
    if _button_caption is None:
        return _variable, _entry, _label
    else:
        _button = Button(_master, text=_button_caption)
        _button.grid(column=2, row=_row, sticky=W)
        return _variable, _entry, _label, _button


class Status_Bar(Frame):
    """
    The status bar visualizes progress and status.
    """
    task = None

    def __init__(self, _master):
        super(Status_Bar, self).__init__(_master, bd=1, relief=SUNKEN)

        self.task = StringVar()
        self.progress = IntVar()
        self.init_widgets()

    def update_task(self, _task, _progress):
        """ This function is used as an endpoint for for the
        bubbling notify_task call of the BPMFrame class.
        :param _task: Text that defines the task
        :param _progress: Text that describes the degree of progress
        """
        if _progress < 0:
            self.pb_progress.mode = 'indeterminate'
        else:
            self.pb_progress.mode = 'determinate'

        self.task.set(_task)
        self.progress.set(_progress)
        self.update_idletasks()

    def init_widgets(self):
        """
        Initialize visual elements.
        """

        self.l_task = ttk.Label(self, textvariable=self.task, width=70)
        self.l_task.pack(side=LEFT, fill=X)

        self.pb_progress = ttk.Progressbar(self, variable=self.progress, length=150)
        self.pb_progress.pack(side=RIGHT)


class Selector(Frame):
    """The Selector class implements a selector, label and variable(to hold the data)
    """
    caption = None
    """The caption of the selector"""
    caption_orientation = None
    """The orientation of the selector, can be "LEFT", "RIGHT", "TOP", "BOTTOM"."""
    value = None
    """The current value of the selector"""
    values = None
    """The selectable values"""

    onchange = None
    """The onchange-event is triggered when the value is changed."""
    do_not_propagate = False
    """If True, the onchange event isn't triggered on changing the value."""


    def __init__(self, _master, _values=None, _caption=None, _caption_orientation=None, _relief=None, _onchange=None):
        super(Selector, self).__init__(_master, bd=1, relief=_relief)

        self.value = StringVar()

        if _values:
            self.values = _values
        else:
            self.values = None

        if _caption:
            self.caption = _caption
        else:
            self.caption = None

        if _caption_orientation:
            self.caption_orientation = _caption_orientation
        else:
            self.caption_orientation = LEFT

        if _onchange:
            self.onchange = _onchange
            # Add handling when type is changed

        else:
            self.onchange = None

        do_not_propagate = False

        self.init_widgets()


    def set_but_do_not_propagate(self, _value):
        """Sets the value, but by setting the do_not_propagate-flag, the onchange event will not fire.
        :param _value: The new value
        """
        self.do_not_propagate = True
        self.value.set(_value)


    def _do_onchange(self, *args):
        """
        Calls onchange if assigned.
        :param args: a list of arguments.
        """
        if self.do_not_propagate == False and self.onchange:
            self.onchange(_current_index=0, _current_value=self.cb.get())
        else:
            self.do_not_propagate = False


    def init_widgets(self):
        """
        Initialize visual elements.
        """

        self.value = StringVar()
        self.cb = ttk.Combobox(self, textvariable=self.value, state='readonly')
        self.cb['values'] = self.values
        self.cb.current(0)

        self.l_caption = ttk.Label(self, text=self.caption)

        _cb_o = None
        if self.caption_orientation == LEFT:
            _cb_o = RIGHT
        elif self.caption_orientation == RIGHT:
            _cb_o = LEFT
        elif self.caption_orientation == TOP:
            _cb_o = BOTTOM
        elif self.caption_orientation == BOTTOM:
            _cb_o = TOP

        self.cb.pack(side=_cb_o)
        self.l_caption.pack(side=self.caption_orientation)

        self.value.trace('w', self._do_onchange)


class VerticalScrolledFrame(Frame):
    """A pure Tkinter scrollable frame that actually works!
    * Use the 'interior' attribute to place widgets inside the scrollable frame
    * Construct and pack/place/grid normally
    * This frame only allows vertical scrolling

    """

    def __init__(self, parent, *args, **kw):
        Frame.__init__(self, parent, *args, **kw)

        # create a canvas object and a vertical scrollbar for scrolling it
        vscrollbar = Scrollbar(self, orient=VERTICAL)
        vscrollbar.pack(fill=Y, side=RIGHT, expand=FALSE)
        canvas = Canvas(self, bd=0, highlightthickness=0,
                        yscrollcommand=vscrollbar.set)
        self._canvas = canvas
        canvas.pack(side=LEFT, fill=BOTH, expand=TRUE)
        vscrollbar.config(command=canvas.yview)

        # reset the view
        canvas.xview_moveto(0)
        canvas.yview_moveto(0)

        # create a frame inside the canvas which will be scrolled with it
        self.interior = interior = Frame(canvas)
        interior_id = canvas.create_window(0, 0, window=interior,
                                           anchor=NW)

        # track changes to the canvas and frame width and sync them,
        # also updating the scrollbar
        def _configure_interior(event):
            # update the scrollbars to match the size of the inner frame
            size = (interior.winfo_reqwidth(), interior.winfo_reqheight())
            canvas.config(scrollregion="0 0 %s %s" % size)
            if interior.winfo_reqwidth() != canvas.winfo_width():
                # update the canvas's width to fit the inner frame
                canvas.config(width=interior.winfo_reqwidth())
            if interior.winfo_reqheight() != canvas.winfo_height():
                # update the canvas's height to fit the inner frame
                canvas.config(height=interior.winfo_reqheight())

        interior.bind('<Configure>', _configure_interior)

        def _configure_canvas(event):
            """
            This function is called when the canvas is configured, that is on resize and so on,
            """
            if interior.winfo_reqwidth() != canvas.winfo_width():
                # update the inner frame's width to fill the canvas
                canvas.itemconfigure(interior_id, width=canvas.winfo_width())
            if interior.winfo_reqheight() != canvas.winfo_height():
                # update the inner frame's height to fill the canvas
                canvas.itemconfigure(interior_id, height=canvas.winfo_height())

        canvas.bind('<Configure>', _configure_canvas)

        return


class BaseFrame(Frame, YView, XView):
    """
    This class introduces all properties that a frame in should hold.
    """

    def notify_task(self, _task, _progress):
        """
        This function checks if the master widget has a notify_task function, and if so, calls it.
        This way, notifications travel upwards in the widget structure until someone has a notify_task that's different.
        See the main_tk_replicator.ReplicatorMain for an example,

        :param _task: Text that defines the task
        :param _progress: Text that describes the degree of progress
        """
        if self.master:
            if hasattr(self.master, "notify_task"):
                self.master.notify_task(_task, _progress)
            else:
                print(
                    self.__class__.__name__ + ": Internal deficiency, " + self.master.__class__.__name__ + " should have a notify_task function\nTask:\n" + _task + "\n" + str(
                        _progress))
        else:
            print("Task:\n" + _task + "\n" + str(_progress))


    def notify_messagebox(self, _title, _message, _kind=None):
        """
        This function checks if the master widget has a notify_messagebox function, and if so, calls it.
        This way, messagebox calls travel upwards in the widget structure until someone has a notify_messagebox that's
        different.
        See the main_tk_replicator.ReplicatorMain for an example,

         :param _title: The title of the message box.
         :param _message: The message to be shown in the text area.
         :param _kind: Can be "message", "warning" or None.
        """

        if self.master:
            if hasattr(self.master, "notify_messagebox"):
                self.master.notify_messagebox(_title, _message, _kind=None)
            else:
                print(
                    self.__class__.__name__ + ":Internal deficiency, " + self.master.__class__.__name__ + " should have a notify_messagebox function.\nTitle:" + _title + "\n" + _message)
        else:
            print("Title:" + _title + "\n" + _message)





class TextExtension(Frame):
    """Extends Frame.  Intended as a container for a Text field.  Better related data handling
    and has Y scrollbar."""


    def __init__(self, master, textvariable=None, *args, **kwargs):

        super(TextExtension, self).__init__(master)
        # Init GUI

        self._y_scrollbar = Scrollbar(self, orient=VERTICAL)

        self._text_widget = Text(self, yscrollcommand=self._y_scrollbar.set, *args, **kwargs)
        self._text_widget.pack(side=LEFT, fill=BOTH, expand=1)


        self._y_scrollbar.config(command=self._text_widget.yview)
        self._y_scrollbar.pack(side=RIGHT, fill=Y)

        if textvariable is not None:
            if not (isinstance(textvariable, Variable)):
                raise TypeError("tkinter.Variable type expected, " + str(type(textvariable)) + " given.".format(type(textvariable)))
            self._text_variable = textvariable
            self.var_modified()
            self._text_trace = self._text_widget.bind('<<Modified>>', self.text_modified)
            self._var_trace = textvariable.trace("w", self.var_modified)

    def text_modified(self, *args):
            if self._text_variable is not None:
                self._text_variable.trace_vdelete("w", self._var_trace)
                self._text_variable.set(self._text_widget.get(1.0, END))
                self._var_trace = self._text_variable.trace("w", self.var_modified)
                self._text_widget.edit_modified(False)

    def var_modified(self, *args):
        self.set_text(self._text_variable.get())
        self._text_widget.edit_modified(False)

    def unhook(self):
        if self._text_variable is not None:
            self._text_variable.trace_vdelete("w", self._var_trace)


    def clear(self):
        self._text_widget.delete(1.0, END)

    def set_text(self, _value):
        self.clear()
        if (_value is not None):
            self._text_widget.insert(END, _value)



"""
This module holds system-global variables for the broker.

Created on Jan 21, 2016

@author: Nicklas Boerjesson
"""

__author__ = 'nibo'

# The states of the processes
states = []
states_lookup = {}

#: Logged in peers
peers = {}




def set_dict_if_set(_dest, _attribute, _value, _default_value):
    """Set a dict attribute if value is set"""
    if _value is not None:
        _dest[_attribute] = _value
    elif _default_value is not None:
        _dest[_attribute] = _default_value

def set_property_if_in_dict(_dest_obj, _property_name, _dict, _default_value = None, _error_msg = None, _convert_underscore=None):
    """Set an object property is the same attribute exists in the dict. If _error is set, raise exception if missing."""

    if _convert_underscore == True:
        _edited = ""
        for _curr_idx in range(0, len(_property_name)):
            if _property_name[_curr_idx -1] == "_":
                _edited+= _property_name[_curr_idx].upper()
            elif _property_name[_curr_idx] != "_":
                _edited+= _property_name[_curr_idx]

        _src_property_name = _edited
    else:
        _src_property_name = _property_name

    if _src_property_name in _dict:
        _dest_obj.__dict__[_property_name] = _dict[_src_property_name]
    elif _default_value is not None:
        _dest_obj.__dict__[_property_name] = _default_value
    elif _error_msg is not None:
        raise Exception(_error_msg + " Attribute missing: " + _property_name)


"""
The setup module handles all Setup related tasks:

Install/remove plugins and their dependencies
Update OF and its dependencies using pip
Manage folder location
Manage startup scripts
Be scriptable
Create a settings file format
Manage databases
Handle certificate (for now, or should they be in the db?)
Update Plugins using git


Deps: dulwich

"""
import os
import sys
import tempfile
from urllib.error import HTTPError, URLError
from urllib.request import urlopen

import json
from distlib.compat import ZipFile
from dulwich import porcelain

from of.common.dictionaries import set_property_if_in_dict
from of.common.internal import not_implemented
from of.common.settings import JSONXPath

default_config_repo = "https://github.com/OptimalBPM/of-config.git"

default_of_install = {
  "installLocation": "~/of/",
  "pluginsFolder": "plugins",
  "plugins": {
    "admin": {
      "url": "https://github.com/OptimalBPM/of-admin.git",
      "branch": "main"
    }
  }
}

class Setup():

    """
    The setup class handles installing and setting up an OF-based system.
    """

    """Location of the setup configuration file"""
    setup_location = None

    """Location of the installation file"""
    install_location = None
    """The location or the plugins, if not set plugins are installed into the config_location/plugins-folder"""
    plugins_folder = None

    """The URL of the config repository"""
    install_repository_url = None

    """An dict of the plugins"""
    plugins = None

    def __init__(self, _setup_definition=None, _setup_filename=None, _default_of_install=False, *args, **kw):

        if _default_of_install:
            self.read_settings(default_of_install)
        elif _setup_definition is not None:
            self.read_settings(_setup_definition)
        elif _setup_filename is not None:
            self.load_from_file(_setup_filename=_setup_filename)

    def load_from_file(self, _setup_filename):
        with open(_setup_filename, "r") as f:
           _setup_definition=json.load(f)

        self.read_settings(_setup_definition)

    def as_dict(self):
        return {"installLocation": self.install_location,
                "pluginsFolder": self.plugins_folder,
                "installRepositoryUrl": self.install_repository_url,
                "plugins": self.plugins}

    def load_install(self, _install_folder):

        _exp_path = os.path.expanduser(_install_folder)
        self.plugins_folder = None

        # Does the config file exist?
        if os.path.exists(os.path.join(_exp_path, "config.json")):
            _config = JSONXPath(os.path.join(_exp_path, "config.json"))
            self.plugins_folder = _config.get_path("broker/pluginsFolder", _default="plugins")

        # TODO: Fetch remote from git remotes
        self.install_repository_url = "Fetching remotes from GIT repo not implemented"

        self.install_location = _install_folder


        if self.plugins_folder is None:
            # If there is no config file, try with the default location for the plugins folder
            self.plugins_folder = os.path.join(_exp_path, "plugins")

        self.plugins = {}


        # Enumerate plugins
        if not os.path.exists(self.plugins_folder):
            # TODO: Write warning to status
            pass
        else:
            # Loading
            _plugin_names = os.listdir(self.plugins_folder)

            for _plugin_name in _plugin_names:
                _curr_plugin_folder = os.path.join(self.plugins_folder, _plugin_name)
                # Only look att non-hidden and non system directories
                if os.path.isdir(_curr_plugin_folder) and _plugin_name[0:2] != "__" and \
                    _plugin_name[0] != ".":
                    _definitions = JSONXPath(os.path.join(_curr_plugin_folder, "definitions.json"))
                    _description = _definitions.get("plugins/" + _plugin_name + "/description","No description found in plugin definition")
                    self.plugins[_plugin_name] = {"name": _plugin_name, "description": _description}


    def read_settings(self, _setup_definition):
        set_property_if_in_dict(self,"install_location", _setup_definition, _convert_underscore=True)
        set_property_if_in_dict(self,"plugins_folder", _setup_definition, _convert_underscore=True, _default_value="plugins")
        set_property_if_in_dict(self, "install_repository_url", _setup_definition,
                                _default_value=default_config_repo, _convert_underscore=True)
        set_property_if_in_dict(self, "plugins", _setup_definition,  _convert_underscore=True)

    @not_implemented
    def uninstall_config(self):
        pass

    @not_implemented
    def uninstall_plugin(self):
        pass

    @not_implemented
    def uninstall(self):
        pass

    def install_config(self):
        # Set folder location at ~/of if not set
        if self.install_location is None:
            _folder_location = os.path.expanduser("~/of")
        else:
            _folder_location = os.path.expanduser(self.install_location)


        if not os.path.exists(_folder_location):
            print("\nInstalling configuration and startup scripts at:\n" +_folder_location)
            print("-----------------------------------------------------")


            # Clone the config repo
            print("Cloning " + self.install_repository_url + " into " + _folder_location)
            _repo = porcelain.clone(source= self.install_repository_url, target= _folder_location, checkout=True)
            _config = _repo.get_config()
            _config.set(('remote "origin"'.encode('ascii'),), "url".encode('ascii'), self.install_repository_url.encode('ascii'))
            _config.write_to_path()
            print("Done.")
        else:
            raise Exception("Error installing the configuration files at \"" + _folder_location + "\", directory already exists.")

    def install_plugin_binaries(self, _folder):
        try:
            with open(os.path.join(_folder, "definitions.json"), "r") as f:
                _settings = json.load(f)
            try:
                for _curr_plugin_name, _curr_plugin_value in _settings["plugins"].items():
                    if "binaries" in _curr_plugin_value:
                        print("\nInstalling and expanding compressed content...")
                        print("----------------------------------------------")
                        for _curr_binary in _curr_plugin_value["binaries"]:
                            _source_file = urlopen(_curr_binary["url"])
                            _tempfile = tempfile.TemporaryFile(mode='w+b')
                            sys.stdout.write("Downloading " + _curr_binary["url"] + " (" + str(_source_file.info()['Content-Length']) + " bytes) into temporary area...")
                            sys.stdout.flush()
                            _tempfile.write(_source_file.read())
                            print("..done.")
                            _target_folder = os.path.join(_folder, _curr_binary["target"])
                            sys.stdout.write("Extracting all files into " + _target_folder)
                            sys.stdout.flush()
                            with ZipFile(_tempfile) as _zipfile:
                                _zipfile.extractall(path=_target_folder)
                            _tempfile.close()
                            print("..done.")

            except HTTPError as e:
                print("An error occured when downloading a binary (" + _curr_binary["url"] + "): " + str(e))
            except URLError as e:
                print("An URL-related error occurred when downloading binary (" + _curr_binary["url"] + "): " + str(e))
        except FileNotFoundError:
            print("No definition file found, ignoring installing plugin binaries in " + _folder)
        except Exception as e:
            print("An error occurred installing binaries in "+ _folder + ":" + str(e))


    def install_plugin(self, _plugins_location, _plugin_name, _plugin_info):
        # TODO: Add branch..
        _curr_target = os.path.join(_plugins_location, _plugin_name)
        print("Cloning "+ _plugin_info["url"] + " into " + _curr_target)
        _repo = porcelain.clone(source=_plugin_info["url"], target=_curr_target, checkout=True)
        _config = _repo.get_config()
        _config.set(('remote "origin"'.encode('ascii'),), "url".encode('ascii'),
                    _plugin_info["url"].encode('ascii'))
        _config.write_to_path()

        print("Done.")

        self.install_plugin_binaries(_curr_target)

    def install_plugins(self):
        # Set plugin location to config location/plugins if not set
        if self.plugins_folder is None:
            _plugins_location = os.path.join(os.path.expanduser(self.install_location), "plugins")

        elif not os.path.isabs(self.plugins_folder):
            _plugins_location = os.path.join(os.path.expanduser(self.install_location), self.plugins_folder)
        else:
            _plugins_location = os.path.expanduser(self.plugins_folder)

        if not os.path.exists(_plugins_location):
            os.mkdir(_plugins_location)

        print("\nInstall plugins at: \n" + _plugins_location)
        for _plugin_name, _plugin_info in self.plugins.items():
            self.install_plugin(_plugins_location, _plugin_name, _plugin_info)

    def install(self):

        self.install_config()
        self.install_plugins()

        # TODO: Fix rights for logging if on linux..
        print("\nSetup is complete, system is installed.")



"""
Created on Oct 20, 2013

@author: Nicklas Boerjesson
"""
import json
import os
import sys
from _thread import start_new_thread

from io import StringIO

from tkinter import Tk, ttk, filedialog, SUNKEN, StringVar, Button, BooleanVar, messagebox, Toplevel, Entry, Text

from tkinter.constants import E, W, N, S, LEFT, X, BOTTOM, TOP, Y, BOTH, RIGHT, END, WORD

from of.tools.setup.lib.frame_list import FrameList
from of.tools.setup.lib.frame_plugin import FramePlugin
from of.tools.setup.lib.widgets_misc import VerticalScrolledFrame, BaseFrame, TextExtension, Selector, Status_Bar, \
    make_entry


class SetupMain(VerticalScrolledFrame):
    """The main class for the GUI of the application"""

    setup = None
    """This is the merge object of the application, it holds all settings for the merge operation"""
    setup_filename = None
    """The name of the file containing the merge definition"""
    fr_src_dataset = None
    """The fram of the source dataset, contains a FrameCustomDataset descendant"""
    fr_dest_dataset = None
    """The fram of the source dataset, contains a FrameCustomDataset descendant"""
    suppress_errors = None
    """Do not show any errors"""
    _row_index = None
    """The current row in the dataset"""
    curr_mapping_frame = None
    """The currently selected mapping frame"""

    def __init__(self, _setup=None, _setup_filename=None, *args, **kw):

        self.parent = Tk()

        # Init oneself

        super(SetupMain, self).__init__(self.parent, bd=1, relief=SUNKEN, *args, **kw)
        self.grid(stick=(E, W, N, S))

        self.suppress_errors = None

        self.setup = _setup
        self.setup_filename = StringVar()
        self.setup_filename.set(_setup_filename)

        self.install_location = None
        self.plugins_folder = None
        self.install_repository_url = None
        self.fr_settings = None

        self.fr_dest_dataset = None

        self.grid()
        self.ip_address = StringVar()
        self._row_index = 0
        self.init_GUI()
        self.notify_task('GUI initiated.', 0)
        if _setup is not None:
            self.notify_task('Loading setup..', 0)
            self.setup_to_gui()
            self.notify_task('Setup loaded.', 100)

        self.parent.columnconfigure(0, weight=1)
        self.parent.rowconfigure(0, weight=1)
        self.resize()
        self.notify_task('Running. Welcome to the Optimal Framework setup application!', 100)
        self.parent.mainloop()


    def resize(self):
        """
        Resize the window, set the width what the internal windows need.
        """
        self._canvas.update_idletasks()
        self.fr_top_right.update_idletasks()

        self._canvas.config(width=self.interior.winfo_reqwidth() + 1, height=self.interior.winfo_reqheight()+1)

        self.fr_bottom.update_idletasks()

    def on_dataset_columns_change(self, *args):
        # Columns have changed; force reload columns from structure
        self.fr_src_dataset.get_possible_references(True)
        self.fr_dest_dataset.get_possible_references(True)
        for curr_mapping in self.g_plugins.items:
            curr_mapping.fr_item.reload_references()


    def notify_task(self, _task, _progress):
        """Override as this is the top widget"""
        self.fr_status_bar.update_task(_task, _progress)

    def notify_messagebox(self, _title, _message, _kind=None):
        """Override as this is the top class, default is error."""
        if self.suppress_errors is None:
            if _kind == "message":
                messagebox.showinfo(_title, _message)
            elif _kind == "warning":
                messagebox.showwarning(_title, _message)
            else:
                messagebox.showerror(_title, _message)

    def on_post_merge_sql(self, *args):
        # Show post-merge-SQL dialog
        _wdw = Toplevel()
        _wdw.geometry('+400+400')
        _wdw.e = TextExtension(_wdw, textvariable=self.post_execute_sql)
        _wdw.e.pack()
        _wdw.e.focus_set()
        _wdw.transient(self.parent)
        _wdw.grab_set()
        self.parent.wait_window(_wdw)
        _wdw.e.unhook()
        del (_wdw)


    def on_select_plugins_folder(self, *args):
        _plugin_folder = filedialog.askdirectory(title="Choose plugin folder (usually in the ")
        if _plugin_folder is not None or _plugin_folder != "":
            self.plugins_folder.set(_plugin_folder)

    def on_select_install_folder(self, *args):
        _install_folder = filedialog.askdirectory(title="Choose install folder (usually in the \"~of\"-folder)")
        if _install_folder is not None or _install_folder != "":
            self.install_location.set(_install_folder)

    def on_select_installation(self, *args):
        _install_folder = filedialog.askdirectory(title="Select existing installation")
        if _install_folder is not None or _install_folder != "":
            self.install_location.set(_install_folder)
            self.setup.load_install(_install_folder=_install_folder)
            self.setup_to_gui()

    def install_thread_function(self, _name, _whatever):
        self.setup.install()

        self.notify_task('Installed.', 100)

    def on_install(self):
        self.gui_to_setup()
        self.notify_task('Installing..', 0)
        start_new_thread(self.install_thread_function, ("Install_thread", None))


    def on_uninstall(self):
        self.gui_to_setup()
        self.notify_task('Uninstalling..', 0)
        self.setup.uninstall()
        self.notify_task('Uninstalled.', 100)


    def intercept_log(self, s):
        self.ta_log.insert("end", s)
        self.ta_log.see(END)
        self.parent.update_idletasks()

        return None

    def init_GUI(self):
        """Init main application GUI"""
        print("Initializing GUI...redirectind stdout/stderr to log window..", end="")


        self.parent.title("Optimal Framework setup")
        self.interior.notify_task = self.notify_task
        self.interior.notify_messagebox = self.notify_messagebox

        self.fr_top = BaseFrame(self.interior)
        self.fr_top.pack(side=TOP, fill=BOTH, expand=1)

        self.fr_top_left = BaseFrame(self.fr_top)
        self.fr_top_left.pack(side=LEFT, fill=BOTH, expand=1)

        self.fr_rw = BaseFrame(self.fr_top_left)
        self.fr_rw.pack(side=TOP, fill=X)

        self.btn_load_json_json = ttk.Button(self.fr_rw, text="Load", command=self.on_load_json)
        self.btn_load_json_json.pack(side=LEFT)
        self.btn_save_json = ttk.Button(self.fr_rw, text="Save", command=self.on_save_json)
        self.btn_save_json.pack(side=LEFT)
        self.btn_load_folder = ttk.Button(self.fr_rw, text="Existing", command=self.on_select_installation)
        self.btn_load_folder.pack(side=LEFT)
        self.fr_setup_filename = BaseFrame(self.fr_rw)
        self.l_setup_filename = ttk.Label(self.fr_setup_filename, text="Setup file:")
        self.l_setup_filename.pack(side=LEFT)
        self.setup_filename.set("")
        self.e_config_filename = ttk.Entry(self.fr_setup_filename, textvariable=self.setup_filename)
        self.e_config_filename.pack(side=RIGHT)

        self.fr_setup_filename.pack(side=RIGHT)

        # Settings

        self.fr_settings = BaseFrame(self.fr_top_left)
        self.fr_settings.pack(side=TOP, fill=BOTH)
        self.fr_settings.columnconfigure(2, weight=5)
        self.install_location,self.l_install_location, self.e_install_location,\
        self.b_install_location = make_entry(self.fr_settings, "Install location:", 0, _button_caption= "..")
        self.b_install_location.config(command = self.on_select_install_folder)
        self.plugins_folder, self.l_plugin_location, self.e_plugin_location, \
        self.b_plugins_folder = make_entry(self.fr_settings, "Plugins folder:", 1, _button_caption="..")
        self.b_plugins_folder.config(command = self.on_select_plugins_folder)
        self.install_repository_url,self.l_install_repository_url, \
        self.e_install_repository_url = make_entry(self.fr_settings, "Repository URL", 2)


        # Plugins

        self.fr_plugins_header = BaseFrame(self.fr_top_left)
        self.fr_plugins_header.pack(side=TOP)

        self.l_plugins = ttk.Label(self.fr_plugins_header, text="Plugins:")
        self.l_plugins.pack(side=TOP)

        self.fr_plugins_header_nav = BaseFrame(self.fr_plugins_header)
        self.fr_plugins_header_nav.pack(side=BOTTOM)



        self.btn_reload = Button(self.fr_plugins_header_nav, text="Refresh", command=self.on_refresh_plugins)
        self.btn_reload.pack(side=LEFT)

        self.g_plugins = FrameList(self.fr_top_left, _detail_key_text="Details >>", bd=1, relief=SUNKEN)
        self.g_plugins.pack(side=TOP, fill=X)
        #self.g_plugins.on_delete = self.plugins_do_on_delete
        #self.g_plugins.on_detail = self.plugins_do_on_detail


        # Plugin details
        self.fr_top_right = BaseFrame(self.fr_top)
        self.fr_top_right.pack(side=RIGHT, fill=Y)

        self.l_plugin = ttk.Label(self.fr_top_right, text="Plugin details")
        self.l_plugin.pack(side=TOP)
        self.fr_plugin = BaseFrame(self.fr_top_right, bd=1, relief=SUNKEN)
        self.fr_plugin.pack(side=TOP, fill=BOTH, expand=1)
        self.plugin_url,self.l_plugin_url, \
        self.e_plugin_url = make_entry(self.fr_plugin, "Repository URL(http):", 0)



        # Install
        self.fr_install = ttk.Frame(self.fr_top_left)
        self.fr_install.pack(side=TOP, fill=BOTH, expand=1)



        self.fr_install_actions = ttk.Frame(self.fr_install)
        self.fr_install_actions.pack(side=TOP, fill=X)

        self.btn_install = Button(self.fr_install_actions, text="Install", command=self.on_install)
        self.btn_install.pack(side=LEFT)
        self.btn_uninstall = Button(self.fr_install_actions, text="Uninstall (not implemented)", command=self.on_uninstall)
        self.btn_uninstall.pack(side=LEFT)



        # Preview

        self.ta_log = Text(self.fr_install, width = 100, height = 7, wrap = WORD)
        self.ta_log.pack(side=TOP, fill=BOTH, expand=1)

        self.log_out = StringIO()
        sys.stdout = self.log_out
        sys.stderr = self.log_out
        self.log_out._write = self.log_out.write
        self.log_out.write = self.intercept_log


        self.fr_bottom = BaseFrame(self.interior)
        self.fr_bottom.pack(side=BOTTOM, fill=BOTH)
        self.fr_bottom.config(height=30)

        self.fr_status_bar = Status_Bar(self.fr_bottom)
        self.fr_status_bar.pack(fill=BOTH)
        self.fr_status_bar.config(height=30)

        self.resize()

    # #########################################################################
    # This section contains functions handling thawe entire setup(load/save/GUI)
    # #########################################################################

    def on_plugin_detail(self, _framelist, _item):
        self.plugin_url.set(_item.fr_item.plugin["url"])

    def plugins_to_gui(self):
        if hasattr(self.setup, "plugins") and self.setup.plugins is not None:
            # clear plugin list
            self.g_plugins.clear()
            self.g_plugins.on_detail = self.on_plugin_detail
            # populate with plugins
            for _curr_plugin_key, _curr_plugin_value in self.setup.plugins.items():
                _new_item = self.g_plugins.append_item()
                _new_item.make_item(_class=FramePlugin, _name = _curr_plugin_key, _plugin=_curr_plugin_value)

    def setup_to_gui(self):
        """
        Populate the GUI from the setup class.
        """

        self.install_location.set(self.setup.install_location)
        self.plugins_folder.set(self.setup.plugins_folder)
        self.install_repository_url.set(self.setup.install_repository_url)
        self.plugins_to_gui()


    def on_save_json(self, *args):
        """Triggered when save-button is clicked.
        Displays a save dialog, fetches GUI data into merge, and saves as JSON into the selected file."""
        self.notify_task('Saving..', 0)
        _filename = filedialog.asksaveasfilename(initialdir=os.path.dirname(self.setup_filename.get()),
                                                 initialfile=os.path.basename(self.setup_filename.get())
                                                 , defaultextension=".json",
                                                 filetypes=[('JSON files', '.json'), ('all files', '.*')],
                                                 title="Choose location")
        if _filename:
            self.gui_to_setup()
            self.notify_task('Saving(Generating JS)..', 0)

            _dict = self.setup.as_dict()
            self.notify_task('Saving(Writing file)..', 50)
            with open (_filename, "w") as _f:
                json.dump(_dict, fp=_f, sort_keys=True, indent=4)

            self.notify_task('Saving..done.', 100)
        else:
            self.notify_task('Saving cancelled.', 0)


    def gui_to_plugin(self):

        self.setup.plugins = {}

        # populate with plugins
        for _curr_plugin in self.g_plugins.items:
            self.setup.plugins[_curr_plugin.fr_item.name] = _curr_plugin.fr_item.gui_to_plugin()

    def gui_to_setup(self):
        """
        Save the the GUI to the setup class.
        """

        self.setup.install_location = self.install_location.get()
        self.setup.plugins_folder = self.plugins_folder.get()
        self.setup.install_repository_url = self.install_repository_url.get()
        self.gui_to_plugin()



    def on_load_json(self, *args):
        """Triggered when load-button is clicked.
        Displays a load dialog, clears the GUI, populates the merge and uppdates the GUI"""
        _setup_filename = filedialog.askopenfilename(defaultextension=".json",
                                               filetypes=[('JSON files', '.json'), ('all files', '.*')],
                                               title="Choose file")
        if _setup_filename is not None:
            self.notify_task('Loading setup..', 0)
            try:
                self.setup.load_from_file(_setup_filename=_setup_filename)
                self.setup_filename.set(_setup_filename)
            except Exception as e:
                self.notify_messagebox("Error loading data", str(e))
            self.setup_to_gui()

            self.notify_task('Loading setup..done', 100)
            self.resize()




    def on_refresh_plugins(self):
        """Triggered when the "Reload data"-button is pressed."""
        self.refresh_plugins()







"""
The utils module gathers all utility functions used in messaging

Created on Jan 22, 2016

@author: Nicklas Boerjesson
"""

import os
import socket
import platform
import datetime
import json
import logging



import requests
import sys


from requests.cookies import RequestsCookieJar

from of.common.logging import write_to_log, EC_NOTIFICATION, SEV_DEBUG, SEV_FATAL, EC_SERVICE, SEV_ERROR, \
    EC_COMMUNICATION
from of.common.messaging.factory import get_current_login
import requests
import urllib3
from urllib3.exceptions import InsecureRequestWarning

# Suppress insecure request
# TODO: However, this chould not always be set
urllib3.disable_warnings(InsecureRequestWarning)

# TODO: get_python_versions should be duplicated into of.common.* (PROD-94)

__author__ = 'Nicklas Borjesson'


def make_log_prefix(_name):
    return _name + "(" + str(os.getpid()) + "):"

def sys_modules():
    result = []
    for curr_key in sys.modules.keys():
        result.append(curr_key)

    return result

def get_environment_data():
    def python_versions():
        _major, _minor, _release, _state, _build = sys.version_info
        return str(_major) + "." + str(_minor) + "." + str(_release) + " " + _state + " build " + str(_build)

    return {
        "hostname": socket.gethostname(),
        "implementation": {
            "language": "python",
            "version": python_versions(),
            "modules": sys_modules(),
        },
        "platform": platform.system(),
        "processor": platform.processor(),
        "systemPid": os.getpid(),
        "user": get_current_login(),
    }

def write_dbg_info(_data):
    write_to_log(_data, _category=EC_NOTIFICATION, _severity=SEV_DEBUG)

def register_at_broker(_address, _type, _server, _username, _password, _log_prefix="", _verify_SSL=True):
    _log_prefix = make_log_prefix(_log_prefix)

    _data = {
        "credentials": {
            "usernamePassword": {
                "username": _username,
                "password": _password
            }
        },
        "environment": get_environment_data(),
        "peerType": _type,
        "address": _address
    }
    write_dbg_info(_log_prefix + "[" + str(datetime.datetime.utcnow()) + "] Registering at broker API.")

    _headers = {'content-type': 'application/json'}
    _response = requests.post(_server + "/register", data=json.dumps(_data), auth=('user', 'pass'), headers=_headers,
                              verify=_verify_SSL)
    if _response.status_code == 500:
        write_dbg_info(_log_prefix + "Broker login failed with internal server error! Exiting.")
        return False
    if _response.status_code != 200:
        write_dbg_info(_log_prefix + "Broker login failed with error + "+ str(_response.status_code) + "! Exiting.")
        return False

    _response_dict = _response.json()
    if _response_dict is not None:
        _data = _response_dict

        if "session_id" in _data:
            write_dbg_info(_log_prefix + "Got a session id:" + _data["session_id"])
            return _data
        else:
            write_to_log(_log_prefix + "Broker login failed! Exiting.", _category=EC_SERVICE, _severity=SEV_ERROR)
        return False

    else:
        write_to_log(_log_prefix + "Broker login failed! Exiting.", _category=EC_SERVICE, _severity=SEV_ERROR)
        return False


def call_api(_url, _session_id, _data, _timeout=None, _print_log=None, _verify_SSL=True):
    """

    :param _url:
    :param _session_id:
    :param _data:
    :param _timeout:
    :param _print_log: Do not call write to log
    :return:
    """

    def do_log(_error, _category=EC_NOTIFICATION, _severity=SEV_DEBUG):
        if _print_log:
            print(_error)
        else:
            write_to_log(_data, _category=_category, _severity=_severity)
        return _error

    _cookie_jar = RequestsCookieJar()
    _cookie_jar.set(name="session_id", value=_session_id, secure=True)

    _headers = {'content-type': 'application/json'}

    _response = requests.post(_url, data=json.dumps(_data), headers=_headers, timeout=_timeout,
                              verify=_verify_SSL, cookies=_cookie_jar)

    _response_dict = None

    if _response.status_code != 200:
        do_log("Response code :" + str(_response.status_code))
        try:
            _response.raise_for_status()
        except Exception as e:
            raise Exception(do_log("Error in call_api:" + str(e), _category=EC_COMMUNICATION, _severity=SEV_ERROR))
    else:
        if _response.content:
            try:
                _response_dict = _response.json()
            except Exception as e:
                do_log("response.content didn't contain JSON data", _category=EC_COMMUNICATION, _severity=SEV_ERROR)
                _response_dict = None

    if _response_dict is not None:
        return _response_dict
    else:
        do_log("Got an empty response from server:" + str(_response.content), _category=EC_COMMUNICATION,
                     _severity=SEV_ERROR)

        return None


def make_error(_code, _message):
    return {
        "error_code": _code,
        "message": _message
    }


def message_is_none(_message, _property, _value):
    """
    If a property is unset, return value, otherwise the specified property

    :param _message: The message
    :param _property: The name of the property
    :param _value: The value
    :return: A value
    """
    if _property in _message:
        return _message[_property]
    else:
        return _value


class MultiprocessingLoggingHandler:
    # TODO: Should this be removed or is it used elsewhere?
    level = 0
    debug_prefix = None

    def handle(self, _record):
        write_dbg_info(self.debug_prefix + ": " + str(_record))

    def __init__(self, _log_prefix=None, _level=None):
        if _log_prefix is not None:
            self.debug_prefix = _log_prefix
        else:
            self.debug_prefix = "No debug_prefix set"

        if _level is not None:
            self.level = _level
        else:
            self.level = logging.INFO



"""
Created on Jan 30, 2014

@author: Nicklas Boerjesson
"""
from tkinter.constants import W, RIGHT, Y, E, X, LEFT, BOTH, N
from tkinter import Tk, ttk, SUNKEN, StringVar, Button, Scrollbar, Canvas

from of.tools.setup.lib.widgets_misc import BaseFrame

__author__ = 'nibo'

class FrameListItem(BaseFrame):
    """This class has all the functionality needed to be part of a list of frames."""
    fr_item = None
    """A sub frame containing the content of the frame."""
    fr_control = None
    """A sub frame with navigator buttons and similar."""
    row__base_class = None
    """The class from which to instantiate the item frame. """

    def __init__(self, _master):
        super(FrameListItem, self).__init__(_master, relief=SUNKEN, bd=1)

        self.init_widgets()

    def make_item(self, _class,  **_create_params):
        """
        Instantiates the item frame.
        :param _class: Class reference
        :param _create_params: The params for the creation
        :return:
        """
        self.fr_item = _class(self, **_create_params)
        self.fr_item.pack(fill=BOTH, pady=5)

    def on_delete(self, *args):
        """If delete has been clicked, tell master to delete self."""
        self.master.do_on_delete(self)
    def on_move_up(self, *args):
        """If move up has been clicked, tell master to move self up."""
        self.master.do_on_move_up(self)
    def on_move_down(self, *args):
        """If move down has been clicked, tell master to move self down."""
        self.master.do_on_move_down(self)

    def on_detail(self):
        """If detail button is clicked, tell master to show details."""
        self.master.do_on_detail(self)

    def init_widgets(self,):

        self.fr_control = BaseFrame(self)
        self.fr_control.pack(side=RIGHT)

        self.btn_delete = Button(self.fr_control, text="del", command=self.on_delete, height = 1)
        self.btn_delete.grid(column=1, row=0, sticky=(N,W))
        self.btn_move_up = Button(self.fr_control, text="△", command=self.on_move_up)
        self.btn_move_up.grid(column=2, row=0, sticky=N)
        self.btn_move_down = Button(self.fr_control, text="▽", command=self.on_move_down)
        self.btn_move_down.grid(column=3, row=0, sticky=N)
        if self.master.detail_key_text:
            self.btn_detail_key = Button(self.fr_control, text=self.master.detail_key_text, command=self.on_detail)
            self.btn_detail_key.grid(column=4, row=0, sticky=N)



class FrameList(Canvas):
    """The FrameList manages a list of FrameListItems.
    """
    items = []
    """List of items"""
    scrollbar = None
    """The scrollbar of the list """
    on_insert = None
    """Event fired before an item is inserted"""
    on_delete = None
    """Event fired before an item is deleted"""
    on_append = None
    """Event fired before an item is appended"""
    on_move_up = None
    """Event fired before an item is moved up"""
    on_move_down = None
    """Event fired before an item is moved down"""

    detail_key_text = None
    """The caption of the detail button"""
    on_detail = None
    """Event fired when the detail button is clicked"""

    def __init__(self, _master, _detail_key_text = None, **kwargs):
        super(FrameList, self).__init__(_master, **kwargs)

        self.scrollbar = None

        self.on_delete = None
        self.on_move_up = None
        self.on_move_down = None
        self.on_detail = None
        self.detail_key_text = _detail_key_text

        self.items = []

    def clear(self):
        """Remove all items from the list"""
        for _curr in self.items:
            _curr.destroy()

        self.items.clear()


    def append_item(self):
        """Append a new item to the list"""
        _new = FrameListItem(self)
        if self.on_append:
            self.on_append(self, _new)
        self.items.append(_new)
        _new.pack(fill=X)
        return _new


    def do_on_delete(self, _item_frame):
        """
        Delete the item from the list.
        :param _item_frame: Item to delete

        """

        if self.on_delete:
            self.on_delete(self, _item_frame)

        self.items.remove(_item_frame)
        _item_frame.destroy()

    def repack_list(self):
        """Repack the list"""
        for _curr_item in self.items:
            _curr_item.pack_forget()

        for _curr_item in self.items:
            _curr_item.pack(fill=X)

    def do_on_move_up(self, _item_frame):
        """
        Move _item_frame one step up in the list
        :param _item_frame: The frame to move
        """

        _curr_idx = self.items.index(_item_frame)
        if _curr_idx > 0:
            if self.on_move_up:
                self.on_move_up(self, _item_frame)

            self.items.insert(_curr_idx-1, self.items.pop(_curr_idx))

            self.repack_list()

    def do_on_move_down(self, _item_frame):
        """
        Move _item_frame one step down in the list
        :param _item_frame: The frame to move
        """
        _curr_idx = self.items.index(_item_frame)
        if _curr_idx < len(self.items)-1:
            if self.on_move_down:
                self.on_move_down(self, _item_frame)

            self.items.insert(_curr_idx+1, self.items.pop(_curr_idx))

            self.repack_list()

    def do_on_detail(self, _item_frame):
        """
        Detail button is clicked in _item_frame, trigger event
        :param _item_frame: The frame to show details for
        """
        if self.on_detail:
            self.on_detail(self, _item_frame)

class FrameCustomItem(BaseFrame):
    """
    This class adds an error display label to the BPM frame, used by item frames in the frame list.
    """

    def __init__(self, _master):
        super(FrameCustomItem, self).__init__(_master)

        self._l_err = None

    def show_error(self, _msg = None):
        """
        Show an error label
        :param _msg: The error message
        """

        if self._l_err is None:
            self._l_err = ttk.Label(self, foreground='red', text = _msg)
            self._l_err.pack()
        else:
            self._l_err.text = _msg


    def hide_error(self):
        """Hide the error label"""
        if self._l_err:
            self._l_err.destroy()
            self._l_err = None


#!/usr/bin/env python3


"""
    ************
    Optimal Setup
    ************
    
    Optimal setup is an application that provides setup services for the Optimal framework.
    
    :copyright: Copyright 2010-2015 by Nicklas Boerjesson
    :license: BSD, see LICENSE for details.
"""

import sys

# Version and release information used by Sphinx for documentation and setuptools for package generation.
from subprocess import CalledProcessError

__version__ = '0.9'
__release__ = '0.9.0'
__copyright__ = '2010-2016, Nicklas Boerjesson'

import os
import json
import getopt

major, minor, micro, releaselevel, serial = sys.version_info


def check_prerequisites():
    if major == 2:
        raise Exception("\nERROR RUNNING INSTALLATION:\n"
                        "You are running the setup with Python " + str(major) + "." + str(minor) + "." + str(
            micro) + "!\n"
                     "Please install Python version 3.4 or higher and run with that version instead (python3 optimalsetup.py)")

    if major == 3 and minor < 4:
        _pythonversion = str(major) + "." + str(minor) + "." + str(micro)

        try:
            print("Python " + _pythonversion + " found, so checking if pip is installed..")
            import pip
            print("it was, continuing..")
        except ImportError:
            print("it wasn't, checking if easy_install is installed..")
            try:

                from setuptools.command import easy_install
                if minor == 3:
                    print("it was, installing pip..")
                    easy_install.main(["pip"])
                else:
                    print("it was, installing pip 7.1.2 because 8.x isn't compatible with 3.0-3.2..")
                    easy_install.main(["pip==7.1.2"])
            except ImportError:
                print("it wasn't, halting installation, raising error.")
                raise Exception("Error RUNNING INSTALLATION:\n"
                                "You are running the setup with Python " + _pythonversion + "!\n"
                                                                                            "It doesn't have pip package management installed as default and this installation failed to install it automatically.\n"
                                                                                            "PLease check the pip web site for instructions for your platform:\n"
                                                                                            "https://pip.pypa.io/en/stable/installing/")


def install_package(_package_name, _arguments=None):
    """
    Install the packages listed if not present
    :param _package_name: The package to install
    :param _argument: An optional argument
    :return:
    """
    _installed = []

    import pip

    _exists = None
    if minor < 3:
        import pkgutil
        _exists = pkgutil.find_loader(_package_name)

    elif minor == 3:
        import importlib
        _exists = importlib.find_loader(_package_name)
    else:
        import importlib
        _exists = importlib.util.find_spec(_package_name)

    if _exists is None:
        print(_package_name + " not installed, installing...")
        if _arguments is None:
            pip.main(['install', _package_name])
        else:
            pip.main(['install', _package_name] + _arguments)
        print(_package_name + " installed...")
        return True
    else:
        print(_package_name + " already installed, skipping...")
        return False


sys.path.append(os.path.join(os.path.dirname(__file__), "../../"))

_help_msg = """
Usage: optimal_setup.py [OPTION]... -d [DEFINITION FILE]... -l [LOG LEVEL]
Set up and alter the Optimal Framework

This stand-alone tool facilitates setting up an installation of the framework

    -d, --definitionfile    Provide the path to an JSON definition file to describe the setup
    -e,                     Initialize editor
    -l, --log_level         Log level
    --default_of_install     Make a default Optimal Framework installation with the admin interface
    
    --help     display this help and exit
    --version  output version information and exit

Always back up your data!

"""


def main():
    """Main program function"""

    _setup_filename = None
    _edit = None
    _log_level = None
    _default_of_install = None

    """Parse arguments"""
    try:
        _opts = None
        _args = None
        _opts, _args = getopt.getopt(sys.argv[1:], "ed:l:", ["help", "version", "default_of_install", "definitionfile=*.json", "log_level="])
    except getopt.GetoptError as err:
        print(str(err) + "\n" + _help_msg + "\n Arguments: " + str(_args))
        sys.exit(2)

    if _opts:
        for _opt, _arg in _opts:

            if _opt == '-e':
                # Do not run the Setup, start the application instead
                _edit = True

            elif _opt in ("-d", "--definitionfile"):
                _setup_filename = _arg
            elif _opt in ("-l", "--log_level"):
                _log_level = _arg
            elif _opt == '--help':
                print(_help_msg)
                sys.exit()
            elif _opt == '--version':
                print(__version__)
                sys.exit()
            elif _opt == "--default_of_install":
                _default_of_install = True

        check_prerequisites()

        """If not installed, install GIT support"""
        install_package("dulwich", ["--global-option=--pure"])
        install_package("distlib")
        install_package("of", "--upgrade")
        # Dynamically import after GIT support is installed
        from of.tools.setup.lib.setup import Setup

        if sys.platform == "windows":
            install_package("win32api")

        if _default_of_install:
            print("Selecting the default installation")
            _setup = Setup(_default_of_install=_default_of_install)
        elif _setup_filename:
            """Load setup"""
            print("Load setup file: " + _setup_filename)

            with open(_setup_filename, "r") as f:
                _setup = Setup(_setup_filename=_setup_filename)

        else:
            """Create empty"""
            _setup = Setup()

        if _log_level:
            _setup.log_level = int(_log_level)

        if not _edit and _default_of_install:
            # Handle headless default install case here for clarity
            _log = _setup.install()
        elif _edit or not _setup_filename:
            """If the users wants to edit or haven't specified a definition file, start the editor"""
            # Bring up the GUI
            if sys.platform == "linux":
                try:
                    import _tkinter
                except ImportError:
                    print("On Linux, the python3-tk package has to be installed:")
                    print("----------------python3-tk installing----------------------------------")
                    from subprocess import check_call
                    try:
                        check_call(["sudo", "apt-get", "install", "python3-tk", "--yes"])
                        print("----------------python3-tk installed----------------------------------")
                    except CalledProcessError:
                        print(
                            "Failed to install the python3-tk-package. Please run \"sudo apt-get install python3-tk\" manually.")
                        exit(1)

            from of.tools.setup.lib.main_tk_setup import SetupMain
            SetupMain(_setup=_setup, _setup_filename=_setup_filename)

        else:
            """Otherwise execute the installation"""
            _log = _setup.install()

        print(str(_log))
    else:
        print("Error: No options provided.\n" + _help_msg)


if __name__ == '__main__':
    main()



"""
Created on Feb 5, 2014

@author: Nicklas Boerjesson
"""
from tkinter import IntVar, StringVar, ttk, BooleanVar
from tkinter.constants import LEFT, X, RIGHT
from of.tools.setup.lib.frame_list import FrameCustomItem

__author__ = 'nibo'


def empty_when_none(_string=None):
    """If _string if None, return an empty string, otherwise return string.
    """
    if _string is None:
        return ""
    else:
        return str(_string)

class FramePlugin(FrameCustomItem):
    """Holds and visualizes a Map between two columns of different datasets"""
    row_index = None

    description = None
    plugins = None

    def __init__(self, _master, _name = None, _plugin = None):
        super(FramePlugin, self).__init__(_master)

        self.name = _name
        # Add monitored variables.

        # Description
        self.description = StringVar()

        self.init_widgets()

        self.plugin = _plugin

        if _plugin is not None:
            self.plugin_to_gui()


    def plugin_to_gui(self):

        if "description" in self.plugin:
            self.description.set(str(empty_when_none(self.plugin["description"])))
        else:
            if "url" in self.plugin:
                self.description.set(self.plugin["url"] + " (Not installed)")
            else:
                raise Exception("Plugin does not have an URL property!")

    def gui_to_plugin(self):

        self.plugin["description"] = self.description.get()
        return self.plugin


    def init_widgets(self):

        """Init all widgets"""

        # Source reference
        self.l_description = ttk.Label(self, textvariable=self.description)
        self.l_description.pack(side=LEFT)
        self.l_description["background"] = "#000000"







